/*******************************************************************************
* Copyright (c)                                                                *
*           2005   Tristan Allwood,                                            *
*           2005   Daniel Burke,                                               *
*           2005   Marc Hull,                                                  *
*           2005   Ekaterina Itskova,                                          *
*           2005   Steve Zymler                                                *
*                                                                              *
* Permission is hereby granted, free of charge, to any person obtaining a      *
* copy of this software and associated documentation files (the                *
* "Software"), to deal in the Software without restriction, including          *
* without limitation the rights to use, copy, modify, merge, publish,          *
* distribute, sublicense, and/or sell copies of the Software, and to           *
* permit persons to whom the Software is furnished to do so, subject to        *
* the following conditions:                                                    *
*                                                                              *
* The above copyright notice and this permission notice shall be included      *
* in all copies or substantial portions of the Software.                       *
*                                                                              *
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS      *
* OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF                   *
* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.       *
* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY         *
* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,         *
* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE            *
* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                       *
*******************************************************************************/
/*
 * Created on 19-Oct-2004
 *
 */
package uk.ac.ic.doc.kevlar.api.programs.arguments;

import uk.ac.ic.doc.kevlar.api.programs.IProgramInfoNode;

/**
 * A base interface for argument info objects. 
 * Use to extract information about any argument. 
 * To find out the specialised type of argument, please use
?* takeVisitor(...).
?* @author Daniel Burke 
 */
public interface IBasicArgumentInfo extends IProgramInfoNode {
    /**
     * @return Returns the argument's name.
     */
    String getName();

    /**
     * 
     * @return true iff the argument must be set when it is available.
     */
    boolean isCompulsary();

    /**
     * 
     * @return the context sensitive help for this argument
     */
    IArgumentHelp getArgumentHelp();

    /**
     * 
     * @return an initial value of the argument. Use the
     *         IBasicArgumentValue::takeVisitor(v) method to find out
     *         specialised type
     */
    IBasicArgumentValue getInitialValue();

    /**
     * Use to do a switch case on the specialised type of the argument
     * 
     * @param visitor
     *            implements the desired logic for each argument type
     */
    void takeVisitor(IArgumentVisitor visitor);
}