/*******************************************************************************
* Copyright (c)                                                                *
*           2005   Tristan Allwood,                                            *
*           2005   Daniel Burke,                                               *
*           2005   Marc Hull,                                                  *
*           2005   Ekaterina Itskova,                                          *
*           2005   Steve Zymler                                                *
*                                                                              *
* Permission is hereby granted, free of charge, to any person obtaining a      *
* copy of this software and associated documentation files (the                *
* "Software"), to deal in the Software without restriction, including          *
* without limitation the rights to use, copy, modify, merge, publish,          *
* distribute, sublicense, and/or sell copies of the Software, and to           *
* permit persons to whom the Software is furnished to do so, subject to        *
* the following conditions:                                                    *
*                                                                              *
* The above copyright notice and this permission notice shall be included      *
* in all copies or substantial portions of the Software.                       *
*                                                                              *
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS      *
* OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF                   *
* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.       *
* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY         *
* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,         *
* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE            *
* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                       *
*******************************************************************************/
/*
 * Created on 19-Oct-2004
 *
 */
package uk.ac.ic.doc.kevlar.api.programs.arguments.internal.iomapping;

import uk.ac.ic.doc.kevlar.api.programs.arguments.IBasicArgumentValue;
import uk.ac.ic.doc.kevlar.api.programs.arguments.IBoolArgumentCondition;
import uk.ac.ic.doc.kevlar.api.programs.arguments.internal.argumentInstance.BoolArgumentValue;
import uk.ac.ic.doc.kevlar.publicUtil.dom.XMLUtil;

/**
 * @author Daniel Burke
 *
 */
public final class BoolArgumentCondition extends AbstractArgumentCondition implements IBoolArgumentCondition {
    private boolean testValue;
    
    public BoolArgumentCondition(String targetArgument, boolean testValue) {
        super(targetArgument);
        this.testValue = testValue;
    }

    /**
     * @return Returns true iff the condition specifies the targetArgument must be true
     */
    public boolean mustBeOn() {
        return testValue;
    }
    
	public boolean isMatch(IBasicArgumentValue value) {
		if(!(value instanceof BoolArgumentValue))
		{
			return false;
		}			
		
		return ((BoolArgumentValue)value).getValue() == testValue;
	}
	
	public String toString()
	{
		return toString(0);
	}
	
	public String toString(int depth)
	{
		StringBuilder text = new StringBuilder();
		
		depth++;
		if(testValue)
		{
			XMLUtil.putElement(text, "bool-true-condition", null, depth);
		} 
		else 
		{
			XMLUtil.putElement(text, "bool-false-condition", null, depth);
		}
		depth--;
		
		return super.toString(depth, text.toString());
	}
}
