/*******************************************************************************
* Copyright (c)                                                                *
*           2005   Tristan Allwood,                                            *
*           2005   Daniel Burke,                                               *
*           2005   Marc Hull,                                                  *
*           2005   Ekaterina Itskova,                                          *
*           2005   Steve Zymler                                                *
*                                                                              *
* Permission is hereby granted, free of charge, to any person obtaining a      *
* copy of this software and associated documentation files (the                *
* "Software"), to deal in the Software without restriction, including          *
* without limitation the rights to use, copy, modify, merge, publish,          *
* distribute, sublicense, and/or sell copies of the Software, and to           *
* permit persons to whom the Software is furnished to do so, subject to        *
* the following conditions:                                                    *
*                                                                              *
* The above copyright notice and this permission notice shall be included      *
* in all copies or substantial portions of the Software.                       *
*                                                                              *
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS      *
* OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF                   *
* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.       *
* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY         *
* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,         *
* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE            *
* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                       *
*******************************************************************************/
package uk.ac.ic.doc.kevlar.api.programs.framework.internal;

import java.io.InputStream;
import java.io.OutputStream;
import java.io.Serializable;
import java.util.Map;

import uk.ac.ic.doc.kevlar.publicInterfaces.IInputPipe;
import uk.ac.ic.doc.kevlar.publicInterfaces.IOutputPipe;

/**
 * @author tora
 */
public interface IPipeManager {

    /**
     * Sets a load of pipes up to be managed.
     * 
     * @param in
     *            The InputStream to multiplex down
     * @param out
     *            The OutputStream to multiplex up
     * @param ins
     *            The InputPipes to manage in
     * @param outs
     *            THe OutputPipes to manage out
     * @param debug
     *            The debug stream to pass across or null if not required
     * @param initialData
     *            Some initial data to push down the outStream, or null if not
     *            required
     * @param cl
     *            A Custom classloader to use with the object input stream, or
     *            null if not required
     * @param fini
     *            A Runnable to run after one of the streams has closed, or null
     *            if none.
     */
    void managePipes(InputStream in, OutputStream out,
            Map<String, IInputPipe> ins, Map<String, IOutputPipe> outs,
            InputStream debug, Serializable initialData, ClassLoader cl,
            Runnable fini);

    /**
     * Blocks until there are no PipeGroups being processed
     * You must not call this in a thread that comes from the execution system (it does bad things)
     */
    void waitTillAllDone();
}
