/*******************************************************************************
* Copyright (c)                                                                *
*           2005   Tristan Allwood,                                            *
*           2005   Daniel Burke,                                               *
*           2005   Marc Hull,                                                  *
*           2005   Ekaterina Itskova,                                          *
*           2005   Steve Zymler                                                *
*                                                                              *
* Permission is hereby granted, free of charge, to any person obtaining a      *
* copy of this software and associated documentation files (the                *
* "Software"), to deal in the Software without restriction, including          *
* without limitation the rights to use, copy, modify, merge, publish,          *
* distribute, sublicense, and/or sell copies of the Software, and to           *
* permit persons to whom the Software is furnished to do so, subject to        *
* the following conditions:                                                    *
*                                                                              *
* The above copyright notice and this permission notice shall be included      *
* in all copies or substantial portions of the Software.                       *
*                                                                              *
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS      *
* OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF                   *
* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.       *
* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY         *
* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,         *
* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE            *
* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                       *
*******************************************************************************/
/*
 * Created on Dec 24, 2004
 */
package uk.ac.ic.doc.kevlar.api.programs.framework.internal;

import java.io.BufferedInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

import uk.ac.ic.doc.kevlar.core.framework.internal.loaders.ExecuteHolder;

/**
 * @author tora
 */
final class RepeatThreadProvider {

    private RepeatThreadProvider() {
        /* protected constructor */
    }
    
    
    public static void makeRepeatThread(InputStream in, OutputStream out) {
        ExecuteHolder.getExecutor().execute(new RepeatThread(in,out));
    }

    
    
    static class RepeatThread implements Runnable{
        
        final InputStream in;
        final OutputStream out;
        
        public RepeatThread( InputStream in, OutputStream out) {
            if ( in instanceof BufferedInputStream ) {
                this.in = in;
            } else {
                this.in = new BufferedInputStream(in);
            }
            
            this.out = out;
        }
        
        
        public void run() {
            Thread.currentThread().setName("Error Stream Repeat Thread");
            int i;

            try {
                while (-1 != (i = in.read())) {
                    out.write(i);
                }
            } catch (IOException ioe) {
                /* do nothing */
            }
        }
    }
}

