/*******************************************************************************
* Copyright (c)                                                                *
*           2005   Tristan Allwood,                                            *
*           2005   Daniel Burke,                                               *
*           2005   Marc Hull,                                                  *
*           2005   Ekaterina Itskova,                                          *
*           2005   Steve Zymler                                                *
*                                                                              *
* Permission is hereby granted, free of charge, to any person obtaining a      *
* copy of this software and associated documentation files (the                *
* "Software"), to deal in the Software without restriction, including          *
* without limitation the rights to use, copy, modify, merge, publish,          *
* distribute, sublicense, and/or sell copies of the Software, and to           *
* permit persons to whom the Software is furnished to do so, subject to        *
* the following conditions:                                                    *
*                                                                              *
* The above copyright notice and this permission notice shall be included      *
* in all copies or substantial portions of the Software.                       *
*                                                                              *
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS      *
* OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF                   *
* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.       *
* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY         *
* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,         *
* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE            *
* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                       *
*******************************************************************************/
/*
 * Created on 03-Nov-2004
 *
 */
package uk.ac.ic.doc.kevlar.api.programs.returnValues;

import java.util.ArrayList;
import java.util.Collection;

import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import uk.ac.ic.doc.kevlar.api.programs.XMLModelLoadingException;
import uk.ac.ic.doc.kevlar.publicUtil.dom.XMLUtil;

/**
 * @author Daniel Burke
 *
 */
public class ReturnModelProvider {
	private ReturnModel backend;
	
	/**
	 * 
	 * @param returnModel the <return-model> element of the XML file
	 * @throws XMLModelLoadingException
	 */
	public ReturnModelProvider(Element returnModel) throws XMLModelLoadingException
	{
	    try
	    {
			Element returnMappingsElem = XMLUtil.yankElement(returnModel, "return-mappings");
			NodeList returnMappingList = returnMappingsElem.getElementsByTagName("return-mapping");
			
			Collection<IReturnMapping> returnMappings = new ArrayList<IReturnMapping>();
			
			for(Element returnMappingElem :  XMLUtil.getElementIterator(returnMappingList))
			{
				returnMappings.add(makeReturnMapping(returnMappingElem));
			}
			
			backend = (ReturnModel) makeReturnModel(returnMappings);
	    }
		catch(Exception e)
		{
		    throw new XMLModelLoadingException(e);
		}
	}
	
	public static IReturnMapping makeReturnMapping(Element returnMappingElem)
	{
		int value = Integer.parseInt(XMLUtil.yankText(returnMappingElem, "value"));
		boolean isGood = null != XMLUtil.yankElement(returnMappingElem, "is-good");
		String message = XMLUtil.yankText(returnMappingElem, "message");
		
		return makeReturnMapping(value, isGood, message);
	}
	
	public ReturnModelProvider(ReturnModel model)
	{
		this.backend = model;
	}
	
	public ReturnModel getReturnModel()
	{
		return backend;
	}
	
	public static IReturnModel makeReturnModel(
			Collection<IReturnMapping> mappings)
	{
		return new ReturnModel(mappings);
	}
	
	public static IReturnMapping makeReturnMapping(
			int value, boolean isGood, String message)
	{
		return new ReturnMapping(value, isGood, message);
	}
	
	public String toString()
	{
		return toString(0);
	}
	
	public String toString(int depth)
	{
		return backend.toString(depth);
	}
}
