/*******************************************************************************
* Copyright (c)                                                                *
*           2005   Tristan Allwood,                                            *
*           2005   Daniel Burke,                                               *
*           2005   Marc Hull,                                                  *
*           2005   Ekaterina Itskova,                                          *
*           2005   Steve Zymler                                                *
*                                                                              *
* Permission is hereby granted, free of charge, to any person obtaining a      *
* copy of this software and associated documentation files (the                *
* "Software"), to deal in the Software without restriction, including          *
* without limitation the rights to use, copy, modify, merge, publish,          *
* distribute, sublicense, and/or sell copies of the Software, and to           *
* permit persons to whom the Software is furnished to do so, subject to        *
* the following conditions:                                                    *
*                                                                              *
* The above copyright notice and this permission notice shall be included      *
* in all copies or substantial portions of the Software.                       *
*                                                                              *
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS      *
* OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF                   *
* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.       *
* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY         *
* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,         *
* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE            *
* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                       *
*******************************************************************************/
package uk.ac.ic.doc.kevlar.core.framework.internal.execute;

import java.util.Set;

import uk.ac.ic.doc.kevlar.api.programs.arguments.IArgumentInstance;
import uk.ac.ic.doc.kevlar.api.programs.framework.internal.execute.ISetupInformation;


/**
 * An implementation of the SetupInformation interface
 * @author tora
 */
class SetupInformation implements ISetupInformation {

    private static final long serialVersionUID = 5903032384890013003L;

    private final Set<String> inputNames;
    private final Set<String> outputNames;
    private final IArgumentInstance args;
    
    SetupInformation(Set<String> inputNames, Set<String> outputNames, IArgumentInstance args){
        this.inputNames = inputNames;
        this.outputNames = outputNames;
        this.args = args;
    }
    
    /**
     * @see uk.ac.ic.doc.kevlar.api.programs.framework.internal.execute.ISetupInformation#getInputPipeNames()
     */
    public Set<String> getInputPipeNames() {
        return inputNames;
    }

    /**
     * @see uk.ac.ic.doc.kevlar.api.programs.framework.internal.execute.ISetupInformation#getOutputPipeNames()
     */
    public Set<String> getOutputPipeNames() {
        return outputNames;
    }

    /**
     * @see uk.ac.ic.doc.kevlar.api.programs.framework.internal.execute.ISetupInformation#getArguments()
     */
    public IArgumentInstance getArguments() {
        return args;
    }

}
