/*******************************************************************************
* Copyright (c)                                                                *
*           2005   Tristan Allwood,                                            *
*           2005   Daniel Burke,                                               *
*           2005   Marc Hull,                                                  *
*           2005   Ekaterina Itskova,                                          *
*           2005   Steve Zymler                                                *
*                                                                              *
* Permission is hereby granted, free of charge, to any person obtaining a      *
* copy of this software and associated documentation files (the                *
* "Software"), to deal in the Software without restriction, including          *
* without limitation the rights to use, copy, modify, merge, publish,          *
* distribute, sublicense, and/or sell copies of the Software, and to           *
* permit persons to whom the Software is furnished to do so, subject to        *
* the following conditions:                                                    *
*                                                                              *
* The above copyright notice and this permission notice shall be included      *
* in all copies or substantial portions of the Software.                       *
*                                                                              *
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS      *
* OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF                   *
* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.       *
* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY         *
* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,         *
* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE            *
* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                       *
*******************************************************************************/
/*
 * Created on 25-Oct-2004 by toa02
 *
 */
package uk.ac.ic.doc.kevlar.core.framework.internal.loaders.directorywatcher;

import java.util.jar.Attributes;
import java.util.jar.JarFile;

/**
 * @author toa02
 *  
 */
public interface INotifierFilter {

    /**
     * Return true if this INotifierFilter filter wants to know about a jar
     * file, using information from the manifest.
     * 
     * @param ats
     *            The Manifest Attributes for the jar file.
     * @return True if this INotifierFilter should be notified about changes to
     *         this jar file.
     */
    boolean hasInterest(Attributes ats);

    /**
     * Called when a JarFile is found that you want to know about. This method
     * is called in thread, holding locks. You should queue the result and use
     * another thread to deal with it.
     * 
     * @param jf
     *            The JarFile that has been added.
     */
    void notifyAdded(JarFile jf);

    /**
     * Called when a JarFile you wanted to know about is removed. This method is
     * called in thread, holding locks. You should queue the result and use
     * another thread to deal with it.
     * 
     * @param jf
     *            The JarFile that has been removed.
     */
    void notifyRemoved(JarFile jf);

    /**
     * Called when a JarFile that you wanted to know about is updated. This
     * method is called in thread, holding locks. You should queue the result and use
     * another thread to deal with it.
     * 
     * @param jf
     *            The JarFile that has been updated.
     */
    void notifyUpdated(JarFile jf);

}
