/*******************************************************************************
* Copyright (c)                                                                *
*           2005   Tristan Allwood,                                            *
*           2005   Daniel Burke,                                               *
*           2005   Marc Hull,                                                  *
*           2005   Ekaterina Itskova,                                          *
*           2005   Steve Zymler                                                *
*                                                                              *
* Permission is hereby granted, free of charge, to any person obtaining a      *
* copy of this software and associated documentation files (the                *
* "Software"), to deal in the Software without restriction, including          *
* without limitation the rights to use, copy, modify, merge, publish,          *
* distribute, sublicense, and/or sell copies of the Software, and to           *
* permit persons to whom the Software is furnished to do so, subject to        *
* the following conditions:                                                    *
*                                                                              *
* The above copyright notice and this permission notice shall be included      *
* in all copies or substantial portions of the Software.                       *
*                                                                              *
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS      *
* OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF                   *
* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.       *
* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY         *
* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,         *
* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE            *
* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                       *
*******************************************************************************/
/*
 * Created on 01-Nov-2004 by toa02
 *
 */
package uk.ac.ic.doc.kevlar.core.programs.testUtils;

import java.util.Map;

import uk.ac.ic.doc.kevlar.api.programs.arguments.IArgumentInstance;
import uk.ac.ic.doc.kevlar.publicInterfaces.AbstractProgram;
import uk.ac.ic.doc.kevlar.publicInterfaces.ICloseEventListener;
import uk.ac.ic.doc.kevlar.publicInterfaces.IInputPipe;
import uk.ac.ic.doc.kevlar.publicInterfaces.IOutputPipe;
import uk.ac.ic.doc.kevlar.publicInterfaces.IPipeEnd;

/**
 * Created for Steve's Test
 * @author sz02
 *
 */
public class ARepeaterProgram extends AbstractProgram{


    /*
     * toa02: Implemented so it actually should do repeat (hopefully)
     */
    public void run(IArgumentInstance args, Map<String,IInputPipe> inputs, Map<String,IOutputPipe> outputs) {
        System.err.println("[Repeater] We are running now");
        
        IInputPipe ip = inputs.get("RepeaterInputPipe");
        IOutputPipe op = outputs.get("RepeaterOutputPipe");
        
        final Thread t = Thread.currentThread();
        
        op.addCloseEventListener(new ICloseEventListener() {
            public void pipeClosed(IPipeEnd source) {
               System.err.println("[Repeater] Pipe Closed Called");
               t.interrupt();
            }
            
        });
        
        try {
            while( !ip.closed() && !op.closed() ) {
                //System.err.println("[Repeater] Entering read-write loop");
                try { 
                    op.write(ip.read());
                } catch ( InterruptedException ie ) {
                    System.err.println("[Repeater] interrupted");
                }
            }
        } catch (IllegalStateException ise ) {
            System.err.println("[Repeater] IllegalState");
        }
        //System.err.println("[Repeater] Fell off the end");
        
        ip.close();
        //System.err.println("IP closed");
        op.close();
        //System.err.println("OP closed");
    }

}
