/*******************************************************************************
* Copyright (c)                                                                *
*           2005   Tristan Allwood,                                            *
*           2005   Daniel Burke,                                               *
*           2005   Marc Hull,                                                  *
*           2005   Ekaterina Itskova,                                          *
*           2005   Steve Zymler                                                *
*                                                                              *
* Permission is hereby granted, free of charge, to any person obtaining a      *
* copy of this software and associated documentation files (the                *
* "Software"), to deal in the Software without restriction, including          *
* without limitation the rights to use, copy, modify, merge, publish,          *
* distribute, sublicense, and/or sell copies of the Software, and to           *
* permit persons to whom the Software is furnished to do so, subject to        *
* the following conditions:                                                    *
*                                                                              *
* The above copyright notice and this permission notice shall be included      *
* in all copies or substantial portions of the Software.                       *
*                                                                              *
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS      *
* OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF                   *
* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.       *
* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY         *
* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,         *
* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE            *
* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                       *
*******************************************************************************/
/*
 * Created on 26-Nov-2004
 *
 */
package uk.ac.ic.doc.kevlar.gui.internal.keyboard.actions;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.KeyEvent;

import uk.ac.ic.doc.kevlar.gui.internal.keyboard.Item;
import uk.ac.ic.doc.kevlar.gui.internal.keyboard.KeyBindings;
import uk.ac.ic.doc.kevlar.gui.internal.keyboard.KeyboardManager;
import uk.ac.ic.doc.kevlar.gui.internal.keyboard.states.MultilineTextBoxState;
import uk.ac.ic.doc.kevlar.gui.internal.keyboard.states.State;
import uk.ac.ic.doc.kevlar.gui.internal.widgets.textbox.MultilineTextBox;

/**
 * @author Marc
 **/
public class MultilineTextBoxAction implements IAction {

    public boolean perform(Item event, State s, KeyEvent e) {

        if (s instanceof MultilineTextBoxState) {
            
            MultilineTextBoxState state = (MultilineTextBoxState)s;
            MultilineTextBox textbox = state.getTextBox();
            
            if (KeyBindings.code("PRESSED").equals(event)) {
            
                if (KeyBindings.code("ESCAPE").equals(e.keyCode)) {
                    
                    KeyboardManager.getSingleton().gotoHistoryState();
                    
                } else if (KeyBindings.code("UP").equals(e.keyCode)) {

                    textbox.getTextArea().moveCaretUp();
                    
                } else if (KeyBindings.code("DOWN").equals(e.keyCode)) {

                    textbox.getTextArea().moveCaretDown();
                    
                } else if (KeyBindings.code("LEFT").equals(e.keyCode)) {
                    
                    if ((e.stateMask & SWT.CTRL) == SWT.CTRL) {
                        
                        textbox.getTextArea().moveCaretWordLeft();
                        
                    } else {
                    
                        textbox.getTextArea().moveCaretLeft();
                        
                    }
                    
                } else if (KeyBindings.code("RIGHT").equals(e.keyCode)) {
                    
                    if ((e.stateMask & SWT.CTRL) == SWT.CTRL) {
                        
                        textbox.getTextArea().moveCaretWordRight();
                        
                    } else {
                        
                        textbox.getTextArea().moveCaretRight();
                        
                    }
                  
                } else if (KeyBindings.code("ENTER").equals(e.keyCode)) {
                    
                    textbox.getTextArea().insert('\n');
                    
                } else if (KeyBindings.code("BACKSPACE").equals(e.keyCode)) {
                    
                    textbox.getTextArea().backSpace();
                    
                } else if (KeyBindings.code("DELETE").equals(e.keyCode)) {
                    
                    textbox.getTextArea().delete();
                    
                } else if (KeyBindings.code("HOME").equals(e.keyCode)) {
                    
                    if ((e.stateMask & SWT.CTRL) == SWT.CTRL) {
                      
                        textbox.getTextArea().ctrlHome();
                    	
                    } else {
                      
                        textbox.getTextArea().home();
                	
                    }
                    
                } else if (KeyBindings.code("END").equals(e.keyCode)) {
                    
                    if ((e.stateMask & SWT.CTRL) == SWT.CTRL) {
                        
                        textbox.getTextArea().ctrlEnd();
                        
                    } else {
                        
                        textbox.getTextArea().end();
                        
                    }
                                          
                } else if (KeyBindings.code("SHIFT_KEY").equals(e.keyCode)) {
                    
                    textbox.getTextArea().startSelect();
                    
                } else if (KeyBindings.code("X").equals(e.keyCode) && (e.stateMask & SWT.CTRL) == SWT.CTRL) {
                    
                    textbox.getTextArea().cut();
                    
                } else if (KeyBindings.code("C").equals(e.keyCode) && (e.stateMask & SWT.CTRL) == SWT.CTRL) {
                    
                    textbox.getTextArea().copy();
                    
                } else if (KeyBindings.code("V").equals(e.keyCode) && (e.stateMask & SWT.CTRL) == SWT.CTRL) {
                    
                    textbox.getTextArea().paste();
                    
                } else if (KeyBindings.isDisplayable(new Item(e.keyCode))) {
                    
                    textbox.getTextArea().insert(e.character);
                    
                } 
                
            } else if (KeyBindings.code("RELEASED").equals(event)) {
                
            	if (KeyBindings.code("SHIFT_KEY").equals(e.keyCode)) {
                
                	textbox.getTextArea().endSelect();
                
            	} 
                
            } 
            
        }
                
        return false;
                
    }

}
