/*******************************************************************************
* Copyright (c)                                                                *
*           2005   Tristan Allwood,                                            *
*           2005   Daniel Burke,                                               *
*           2005   Marc Hull,                                                  *
*           2005   Ekaterina Itskova,                                          *
*           2005   Steve Zymler                                                *
*                                                                              *
* Permission is hereby granted, free of charge, to any person obtaining a      *
* copy of this software and associated documentation files (the                *
* "Software"), to deal in the Software without restriction, including          *
* without limitation the rights to use, copy, modify, merge, publish,          *
* distribute, sublicense, and/or sell copies of the Software, and to           *
* permit persons to whom the Software is furnished to do so, subject to        *
* the following conditions:                                                    *
*                                                                              *
* The above copyright notice and this permission notice shall be included      *
* in all copies or substantial portions of the Software.                       *
*                                                                              *
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS      *
* OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF                   *
* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.       *
* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY         *
* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,         *
* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE            *
* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                       *
*******************************************************************************/
/*
 * Created on 08-Nov-2004
 */
package uk.ac.ic.doc.kevlar.gui.internal.layout;

import org.eclipse.swt.graphics.Point;

/**
 * Interface defining the call-back methods a layout
 * can use to set the path of a pipe.
 * 
 * @author Marc
 **/
public interface ILayoutPipe {

    /**
     * Allows a layout to remove all points in the
     * pipe's path.
     */
    public void removePoints();
    
    /**
     * Allows a layout to add a new point to the path
     * of the pipe. Points are connected together by
     * straight lines to form the graphical representation
     * of the path between two nodes.
     * 
     * @param point the new path point to be added
     */
    public void addPoint(Point point);
    
    /**
     * Returns the source ILayoutProgram that this pipe
     * is currently connected to.
     * 
     * @return the source ILayoutProgram
     */
    public ILayoutProgram getSource();
    
    /**
     * Returns the destination ILayoutProgram that this pipe
     * is currently connected to.
     * 
     * @return the destination ILayoutProgram
     */
    public ILayoutProgram getDestination();
        
    /**
     * Returns the array of points that make up this pipe
     * 
     * @return the array of points
     */
    public Point[] getPoints();
    
}
