/*******************************************************************************
* Copyright (c)                                                                *
*           2005   Tristan Allwood,                                            *
*           2005   Daniel Burke,                                               *
*           2005   Marc Hull,                                                  *
*           2005   Ekaterina Itskova,                                          *
*           2005   Steve Zymler                                                *
*                                                                              *
* Permission is hereby granted, free of charge, to any person obtaining a      *
* copy of this software and associated documentation files (the                *
* "Software"), to deal in the Software without restriction, including          *
* without limitation the rights to use, copy, modify, merge, publish,          *
* distribute, sublicense, and/or sell copies of the Software, and to           *
* permit persons to whom the Software is furnished to do so, subject to        *
* the following conditions:                                                    *
*                                                                              *
* The above copyright notice and this permission notice shall be included      *
* in all copies or substantial portions of the Software.                       *
*                                                                              *
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS      *
* OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF                   *
* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.       *
* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY         *
* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,         *
* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE            *
* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                       *
*******************************************************************************/
/*
 * Created on 08-Nov-2004
 */
package uk.ac.ic.doc.kevlar.gui.internal.layout;

import org.eclipse.swt.graphics.Point;

/**
 * Interface defining the call-back methods that
 * a layout can use to set the position of a program.
 * 
 * @author Marc
 **/
public interface ILayoutProgram {

    /**
     * Allows the layout to access the current position
     * of this program.
     * 
     * @return the position of the program
     */
    public Point getPosition();
    
    /**
     * Allows the layout to access the current size of
     * this program.
     * 
     * @return the size of the program
     */
    public Point getSize();
    
    /**
     * Allows the layout to set the position of the program.
     * 
     * @param position the position to which the program
     * should be moved
     */
    public void setPosition(Point position);
    
    /**
     * Allows the layout to add layout-specific information
     * to a program. This will be saved and loaded with the
     * rest of the pipeline data.
     * 
     * @param data the layout data to be attached
     */
    public void setLayoutData(ILayoutData data);
    
    /**
     * Allows the layout to retrieve previously added layout
     * data, or null if none exists.
     * 
     * @return previously set layout data or null if none
     * exists
     */
    public ILayoutData getLayoutData();
    
}
