/*******************************************************************************
* Copyright (c)                                                                *
*           2005   Tristan Allwood,                                            *
*           2005   Daniel Burke,                                               *
*           2005   Marc Hull,                                                  *
*           2005   Ekaterina Itskova,                                          *
*           2005   Steve Zymler                                                *
*                                                                              *
* Permission is hereby granted, free of charge, to any person obtaining a      *
* copy of this software and associated documentation files (the                *
* "Software"), to deal in the Software without restriction, including          *
* without limitation the rights to use, copy, modify, merge, publish,          *
* distribute, sublicense, and/or sell copies of the Software, and to           *
* permit persons to whom the Software is furnished to do so, subject to        *
* the following conditions:                                                    *
*                                                                              *
* The above copyright notice and this permission notice shall be included      *
* in all copies or substantial portions of the Software.                       *
*                                                                              *
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS      *
* OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF                   *
* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.       *
* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY         *
* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,         *
* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE            *
* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                       *
*******************************************************************************/
/*
 * Created on 18-Nov-2004
 *
 */
package uk.ac.ic.doc.kevlar.gui.internal.pipeline;

import uk.ac.ic.doc.kevlar.api.programs.arguments.IBasicArgumentInfo;
import uk.ac.ic.doc.kevlar.api.programs.arguments.IInputNode;
import uk.ac.ic.doc.kevlar.api.programs.arguments.IOutputNode;
import uk.ac.ic.doc.kevlar.gui.GUIManager;
import uk.ac.ic.doc.kevlar.gui.internal.widgets.listbox.ListBox;
import uk.ac.ic.doc.kevlar.gui.internal.widgets.listbox.ListEntry;
import uk.ac.ic.doc.kevlar.hial.IArgumentValuePair;
import uk.ac.ic.doc.kevlar.hial.IHialProgram;

/**
 * @author Marc
 **/
public class ProgramEntryListBox extends ListBox {

    private static final String ARGUMENT_SKIN = "argumentheading";
    private static final String INPUT_SKIN = "inputheading";
    private static final String OUTPUT_SKIN = "outputheading";
    
    protected Program program;
    
    public ProgramEntryListBox(Program p) {
        
        program = p;
        setSelectionListener(program);
        
        if (p instanceof Macro) {
            
            setData(p.getNodes());
            
        } else {
        
            IHialProgram hialProgram = GUIManager.getHialContext().getProgram(program);
        
            if (hialProgram != null) {
        
                setData(hialProgram.getAllProgramInfoNodes());
            
            }
            
        }
        
    }
    
    public void entrySelected(Object entry) {
        
        super.entrySelected(entry);
        setData(GUIManager.getHialContext().getProgram(program).getAllProgramInfoNodes());
        
    }
        
    public ListEntry createEntry(Object o, int index) {
        
        if (o instanceof IBasicArgumentInfo) {
            
            return new ProgramEntryListEntry(this, index, (IBasicArgumentInfo)o, ARGUMENT_SKIN);
            
        } else if (o instanceof IArgumentValuePair) {
            
            return new ProgramEntryListEntry(this, index, (IArgumentValuePair)o, ARGUMENT_SKIN);
            
        } else if (o instanceof IInputNode) {
            
            return new ProgramEntryListEntry(this, index, (IInputNode)o, INPUT_SKIN);
            
        } else if (o instanceof IOutputNode) {
            
            return new ProgramEntryListEntry(this, index, (IOutputNode)o, OUTPUT_SKIN);
            
        } else if (o instanceof InputNodeEntry) {
            
            return new ProgramEntryListEntry(this, index, (InputNodeEntry)o, INPUT_SKIN);
            
        } else if (o instanceof OutputNodeEntry) {
            
            return new ProgramEntryListEntry(this, index, (OutputNodeEntry)o, OUTPUT_SKIN);
            
        }         
        
        return null;
        
    }
        
/*    protected void update() {
    	
        contents.removeAllChildren();
        entries.clear();
        int positionIndex = 0;
        
    	for (int i=0; i<data.size(); i++) {
    	    
    	    Object o = data.get(i);
    	    ListEntry entry = null;
    	    
    	    if (o instanceof IBasicArgumentInfo) {
    	    
    	        entry = new ProgramListEntry(this, positionIndex, (IBasicArgumentInfo)o, ARGUMENT_SKIN);
    	        
    	    } else if (o instanceof IArgumentValuePair) {
            	    
       	        entry = new ProgramListEntry(this, positionIndex, (IArgumentValuePair)o, ARGUMENT_SKIN);
            	        
    	    } else if (o instanceof IInputNode) {

    	        entry = new ProgramListEntry(this, positionIndex, (IInputNode)o, INPUT_SKIN);
    	        
    	    } else if (o instanceof IOutputNode) {

    	        entry = new ProgramListEntry(this, positionIndex, (IOutputNode)o, OUTPUT_SKIN);
    	        
    	    } 
    	    
    	    if (entry != null) {
    	    
    	        entries.add(entry);
    	        addChild(entry);
    	        positionIndex++;
    	        
    	    }
    	    
    	}
    	
    } */
    
}
