/*******************************************************************************
* Copyright (c)                                                                *
*           2005   Tristan Allwood,                                            *
*           2005   Daniel Burke,                                               *
*           2005   Marc Hull,                                                  *
*           2005   Ekaterina Itskova,                                          *
*           2005   Steve Zymler                                                *
*                                                                              *
* Permission is hereby granted, free of charge, to any person obtaining a      *
* copy of this software and associated documentation files (the                *
* "Software"), to deal in the Software without restriction, including          *
* without limitation the rights to use, copy, modify, merge, publish,          *
* distribute, sublicense, and/or sell copies of the Software, and to           *
* permit persons to whom the Software is furnished to do so, subject to        *
* the following conditions:                                                    *
*                                                                              *
* The above copyright notice and this permission notice shall be included      *
* in all copies or substantial portions of the Software.                       *
*                                                                              *
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS      *
* OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF                   *
* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.       *
* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY         *
* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,         *
* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE            *
* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                       *
*******************************************************************************/
/*
 * Created on 19-Nov-2004
 *
 */
package uk.ac.ic.doc.kevlar.gui.internal.pipeline.arguments;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.widgets.Combo;

import uk.ac.ic.doc.kevlar.gui.internal.AMouseListenerObject;
import uk.ac.ic.doc.kevlar.gui.internal.caches.Utils;
import uk.ac.ic.doc.kevlar.gui.internal.keyboard.KeyboardManager;
import uk.ac.ic.doc.kevlar.gui.internal.keyboard.states.EditAttributeState;
import uk.ac.ic.doc.kevlar.gui.internal.popup.PopUpManager;
import uk.ac.ic.doc.kevlar.gui.internal.widgets.swt.ISWTOkListener;
import uk.ac.ic.doc.kevlar.gui.internal.widgets.swt.SWTComboBox;
import uk.ac.ic.doc.kevlar.publicUtil.collections.IImmutableSet;

/**
 * @author Marc
 **/
public class SetArgumentEditor extends AMouseListenerObject implements ISWTOkListener {

    private static final String EDIT_BOX_SKIN = "editbox";
    private static final Point DEFAULT_EDIT_BOX_SIZE = new Point(200, 30);
    
    protected SWTComboBox combobox;
    protected SetArgument argument;
    
    public SetArgumentEditor(SetArgument arg) {
        
        super();
        argument = arg;
        setSize(DEFAULT_EDIT_BOX_SIZE);
        
    }
    
    public void show() {
        
        if (combobox == null) {
            
            combobox = new SWTComboBox(SWT.DROP_DOWN | SWT.READ_ONLY);
            combobox.setPosition(new Point(5, 2));
            combobox.setSizeFillBoth(new Point(2, 5));
            
            IImmutableSet<String> keySet = argument.getArgumentInfo().getSetValues().keySet();
            Combo combo = combobox.getComboBox();            
            int i = 0;
            
            for (String s : keySet) {
                
                combo.add(s);
                
                if (s.equals(argument.getValueData().getValue())) {
                    
                    combo.select(i);
                    
                }
                
                i++;
                
            }
            
            combobox.setOkListener(this);
            addChild(combobox);            
            
            KeyboardManager.getSingleton().setState(new EditAttributeState(KeyboardManager.getSingleton().getState(), this));
                        
        }
        
    }
    
    public void hide() {
        
        super.hide();

        argument.setValue(combobox.getComboBox().getText());
        
        combobox.dispose();
        combobox = null;
        
    }
    
    public void drawBefore(GC gc, Rectangle redrawArea) {
        
        Utils.drawBox(gc, new Rectangle(area.x-redrawArea.x, area.y-redrawArea.y, area.width, area.height), EDIT_BOX_SKIN);
        
    }

    public void onOk() {
        
        PopUpManager.getSingleton().hidePopUp(this);
        
    }

}
