/*******************************************************************************
* Copyright (c)                                                                *
*           2005   Tristan Allwood,                                            *
*           2005   Daniel Burke,                                               *
*           2005   Marc Hull,                                                  *
*           2005   Ekaterina Itskova,                                          *
*           2005   Steve Zymler                                                *
*                                                                              *
* Permission is hereby granted, free of charge, to any person obtaining a      *
* copy of this software and associated documentation files (the                *
* "Software"), to deal in the Software without restriction, including          *
* without limitation the rights to use, copy, modify, merge, publish,          *
* distribute, sublicense, and/or sell copies of the Software, and to           *
* permit persons to whom the Software is furnished to do so, subject to        *
* the following conditions:                                                    *
*                                                                              *
* The above copyright notice and this permission notice shall be included      *
* in all copies or substantial portions of the Software.                       *
*                                                                              *
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS      *
* OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF                   *
* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.       *
* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY         *
* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,         *
* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE            *
* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                       *
*******************************************************************************/
/*
 * Created on 20.10.2004
 *
 */
package uk.ac.ic.doc.kevlar.gui.internal.taskpane.directorypane;

import java.io.FileNotFoundException;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;

import uk.ac.ic.doc.kevlar.core.framework.internal.execute.ExecutionEngineProvider;
import uk.ac.ic.doc.kevlar.gui.internal.caches.ColorCache;
import uk.ac.ic.doc.kevlar.gui.internal.caches.FontCache;
import uk.ac.ic.doc.kevlar.gui.internal.caches.ImageCache;
import uk.ac.ic.doc.kevlar.gui.internal.taskpane.ATaskPane;

/**
 * @author ei02
 **/

public class DirectoryEntry extends ADirectoryTreeNodeEntry{
	
	static final String CHILD_CATEGORY_BACKGROUND_FILENAME = "directory.png";
	static final String CHILD_CATEGORY_ROLLOVER_FILENAME = "directory_rollover.png";
	
	public DirectoryEntry(IDirectoryNode child, int panePositionIndex, ATaskPane programPane) {
	    
		super(child, panePositionIndex, programPane);
		initialise();

	}
	
	public DirectoryEntry(IDirectoryNode child, int panePositionIndex, ATaskPane programPane, int animationMode) {
	    
		super(child, panePositionIndex, programPane, animationMode);
		initialise();

	}	
		
	public void initialise() {
		
		try {
			
			this.background = ImageCache.get(CHILD_CATEGORY_BACKGROUND_FILENAME);
			
		} catch (FileNotFoundException e) {
			
			this.background = null;
			
		}	    
	    
		try {
			
			this.rollover = ImageCache.get(CHILD_CATEGORY_ROLLOVER_FILENAME);
			
		} catch (FileNotFoundException e) {
			
			this.rollover = null;
			
		}	    
	    
	}

	public void drawBefore(GC gc, Rectangle area) {
	    
	    gc.setForeground(ColorCache.get(0, 0, 0));
	    gc.setFont(FontCache.get("Arial", 8, SWT.BOLD));
		super.drawBefore(gc, area);
		
	}

	public OpenDirectoryEntry convertToParent() {
	    
	    OpenDirectoryEntry result = new OpenDirectoryEntry((IDirectoryNode)dataNode, index, programPane, NO_ANIMATION);
	    programPane.addChild(result);
	    remove();
	    return result;
	    
	}
	
	public void mousePress(Point position, int mouseButton) {
	    
		if (mouseButton==1) {
		    
		    perform();
		    
		}
		
	}
	
	public void perform() {
	    
	    if (programPane instanceof ALoadSavePane)
	    {
	        /*String dirSeparator = dataNode.getFile().separator;
	        String dirPath = dataNode.getFile().getAbsolutePath()+dirSeparator;
	        ((ALoadSavePane)programPane).setCurrentDirectory(dirPath); */
	        
	        ((ALoadSavePane)programPane).setCurrentDirectory(dataNode, this);
	    }
	    else if (programPane instanceof DirectoryPane)
	    {
	        ExecutionEngineProvider.getExecutionEngine().setWorkingDirectory(dataNode.getFile());
	    }
	    
	    ((ADirectoryBrowsePane)programPane).refresh((IDirectoryNode)dataNode, SLIDE_LEFT);
		//ExecutionEngineProvider.getExecutionEngine().setWorkingDirectory(dataNode.getFile());
			
	}

}
