/*******************************************************************************
* Copyright (c)                                                                *
*           2005   Tristan Allwood,                                            *
*           2005   Daniel Burke,                                               *
*           2005   Marc Hull,                                                  *
*           2005   Ekaterina Itskova,                                          *
*           2005   Steve Zymler                                                *
*                                                                              *
* Permission is hereby granted, free of charge, to any person obtaining a      *
* copy of this software and associated documentation files (the                *
* "Software"), to deal in the Software without restriction, including          *
* without limitation the rights to use, copy, modify, merge, publish,          *
* distribute, sublicense, and/or sell copies of the Software, and to           *
* permit persons to whom the Software is furnished to do so, subject to        *
* the following conditions:                                                    *
*                                                                              *
* The above copyright notice and this permission notice shall be included      *
* in all copies or substantial portions of the Software.                       *
*                                                                              *
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS      *
* OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF                   *
* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.       *
* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY         *
* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,         *
* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE            *
* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                       *
*******************************************************************************/
/*
 * Created on 18-Nov-2004
 *
 */
package uk.ac.ic.doc.kevlar.gui.internal.widgets.swt;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.widgets.Control;

import uk.ac.ic.doc.kevlar.gui.internal.AMouseListenerObject;
import uk.ac.ic.doc.kevlar.gui.internal.CanvasManager;
import uk.ac.ic.doc.kevlar.gui.internal.keyboard.KeyboardManager;

/**
 * @author Marc
 **/
public class SWTControl extends AMouseListenerObject implements KeyListener {

    protected Control swtControl;
    protected ISWTOkListener listener;
    
    public SWTControl(Control swtControl) {
        
        super();
        this.swtControl = swtControl;
        this.swtControl.addKeyListener(KeyboardManager.getSingleton());
        this.swtControl.addTraverseListener(KeyboardManager.getSingleton());
//        this.swtControl.addKeyListener(this);
        this.swtControl.forceFocus();
        
    }
    
    public void updateDimensions() {
        
        super.updateDimensions();
        
        if (!swtControl.isDisposed()) {
        
            Rectangle absoluteArea = getAbsoluteArea();
            swtControl.setLocation(absoluteArea.x, absoluteArea.y);
            swtControl.setSize(absoluteArea.width, absoluteArea.height);
            
        }
        
    }
    
    public void hide() {
        
        super.hide();
        
        if (!swtControl.isDisposed()) {
        
            swtControl.setVisible(false);
            
        }
        
        CanvasManager.getSingleton().getFocus();
        
    }
    
    public void show() {
        
        super.show();
        
        if (!swtControl.isDisposed()) {
        
            swtControl.setVisible(true);
            
        }
        
    }
    
    public void dispose() {
        
        if (!swtControl.isDisposed()) {
        
            swtControl.dispose();
            
        }
        
    }
    
    public void setOkListener(ISWTOkListener l) {
        
        listener = l;
        
    }

    public void keyPressed(KeyEvent arg0) {

        if (arg0.character == SWT.ESC || arg0.keyCode == 13) {
            
            listener.onOk();
            
        }
        
    }

    public void keyReleased(KeyEvent arg0) {
        
    }

}
