/*******************************************************************************
* Copyright (c)                                                                *
*           2005   Tristan Allwood,                                            *
*           2005   Daniel Burke,                                               *
*           2005   Marc Hull,                                                  *
*           2005   Ekaterina Itskova,                                          *
*           2005   Steve Zymler                                                *
*                                                                              *
* Permission is hereby granted, free of charge, to any person obtaining a      *
* copy of this software and associated documentation files (the                *
* "Software"), to deal in the Software without restriction, including          *
* without limitation the rights to use, copy, modify, merge, publish,          *
* distribute, sublicense, and/or sell copies of the Software, and to           *
* permit persons to whom the Software is furnished to do so, subject to        *
* the following conditions:                                                    *
*                                                                              *
* The above copyright notice and this permission notice shall be included      *
* in all copies or substantial portions of the Software.                       *
*                                                                              *
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS      *
* OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF                   *
* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.       *
* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY         *
* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,         *
* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE            *
* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                       *
*******************************************************************************/
/*
 * Created on 22-Dec-2004
 *
 */
package uk.ac.ic.doc.kevlar.gui.internal.widgets.textbox;

import org.eclipse.swt.graphics.Point;

import uk.ac.ic.doc.kevlar.gui.internal.ADrawableObject;
import uk.ac.ic.doc.kevlar.gui.internal.AMouseListenerObject;
import uk.ac.ic.doc.kevlar.gui.internal.widgets.IScrollingWindow;
import uk.ac.ic.doc.kevlar.gui.internal.widgets.scrollbars.AScrollbar;
import uk.ac.ic.doc.kevlar.gui.internal.widgets.scrollbars.VerticalScrollBar;

/**
 * @author Marc
 **/
public class ScrollingTextArea extends AMouseListenerObject implements IScrollingWindow {

    protected TextArea contents;
    protected VerticalScrollBar vertScrollbar;
    
    public ScrollingTextArea(MultilineTextBox parent) {
        
        super();
        
        vertScrollbar = new VerticalScrollBar(this);
        vertScrollbar.setIncrement(10);
        addChild(vertScrollbar);
        
        contents = new TextArea(parent);
        addChild(contents);
        
    }
    
    public TextArea getTextArea() {
        
        return contents;
        
    }
    
    public void childDimensionsChanged(ADrawableObject child) {
        
        if (child == contents) {
            
            vertScrollbar.setMaximum(contents.getSize().y);
            
        }
        
    }
        
    public void scrollTo(int y) {
        
        int curY = -contents.getPosition().y;
        int curH = getSize().y;
        
        if (y < curY) {
            
	        contents.setPosition(new Point(contents.getPosition().x, -y));
	        vertScrollbar.updateFromWindow();
            
        } else if (y+20 > curY+curH) {
            
	        contents.setPosition(new Point(contents.getPosition().x, -(y-curH+20)));
	        vertScrollbar.updateFromWindow();
            
        }
        
    }
    
    public void setVerticalPosition(int position) {

        contents.setPosition(new Point(contents.getPosition().x, -position));
        redraw();
        applyRedraw();
        
//        updateScrollbarVisibility();
        
    }

    public void setHorizontalPosition(int position) {

        /* Not used */
        
    }

    public int getVerticalPosition() {
        
        return -contents.getPosition().y;
        
    }

    public int getHorizontalPosition() {
        
        return 0;
        
    }

    private void updateScrollbarVisibility() {
        
	    if (vertScrollbar != null) {
	        
	       	if (vertScrollbar.isUsingFullSpace()) {
	       	    
	       		if (vertScrollbar.isVisible()) {
	       		    
		       		vertScrollbar.hide();
		       		
	       		}
	       		
	    	} else {
	    	    
	       		if(!vertScrollbar.isVisible()) {
	       		    
		    		vertScrollbar.show();
		    		
	       		}
	       		
	    	}
	       	
	    }
	    
    }
       
    public void updateDimensions() {
        
        super.updateDimensions();
        
        contents.setSize(new Point(area.width-AScrollbar.SCROLLBAR_SIZE, 0));
        vertScrollbar.setSize(area.height);
        vertScrollbar.setMaximum(contents.getSize().y);
        vertScrollbar.updateFromWindow();        
//        updateScrollbarVisibility();
        
    }
        
}
